//-----------------------------------------------------------------------------
// File:     HelloMath3.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Tests out various mathematical functions
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
#include <iostream>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <fstream>
using namespace std;


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
double ConvertFromRadiansToDegrees( long angleInDegrees) { return angleInDegrees * 0.017453292519943295769236907684886127134428718885417254560971914402; }
double ConvertFromDegreesToRadians( long angleInRadians) { return angleInRadians * 57.295779513082320876798154814105170332405472466564321549160243861; }
bool   WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr );
bool  GetStringFromFile( char inputString[], unsigned long sizeOfString, FILE* fptr);
bool  GetStringFromKeyboard( char inputString[], unsigned long sizeOfString);
bool  WriteStringToScreen ( const char outputString[]);
bool  WriteStringToFile( const char outputString[], FILE *fptr);
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue);
FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute);FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute);
bool GetDoubleRowElementsFromFile(double *array, unsigned int numberOfCols, FILE *fptr);
bool GetDoubleMatrixElementsFromFile(double *array, unsigned int numberOfRows, unsigned int numberOfCols, FILE *fptr);
bool WriteDoubleRowElemementsToFile(const double *array, unsigned int numberOfCols, int precision, FILE *fptr);
bool WriteDoubleMatrixELementsToFile(const double *array, unsigned int numberOfRows, unsigned int numberOfCols, int precision, FILE *fptr);
void AddMatrics(const double *arrayA, const double *arrayB, double *arraySum, unsigned int numberOfElements);
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{  	
   const char fileAttributeW[5]="w";
	
   //Function opens file for reading and returns error is unable to open 
   FILE *pFile = FileOpenWithMessageIfCannotOpen("HelloMath3In.txt","rb");
	if (!pFile) return 1;
	
   //Creates an array that will be used to store the numbers of HelloMath3 
   double mArray[35];
	double *Parray=mArray;
   
   //Variables with the number of rows and columns for the matrix
	int numberOfRows=5, numberOfCols=7;
	
   //Fumction retrieves numbers from HelloMath3In.txt
   GetDoubleMatrixElementsFromFile(Parray,numberOfRows,numberOfCols,pFile);
	
   //Creates variable that will store the sum of mArray+mArray
   double arraySum[35];
	double *ParraySum=arraySum;

   //Function adds the two matrices together
	AddMatrics(mArray, mArray, Parray, 35);

	//Function Opens a file called HelloMath3Out.txt and writes computed matrix (arraySum) to it
   FILE *pFileout = FileOpenWithMessageIfCannotOpen("HelloMath3Out.txt","w");
   WriteDoubleMatrixELementsToFile(Parray, 5, 7, 8, pFileout); 
   
   // Keep the screen displayed until the user presses the Enter key
   WriteStringToScreen( "\n\n Press  Enter  to terminate the program: " );
   int key = getchar();
   
   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;
}


//------------------------------------------------------------------------------------------------
bool WriteDoubleMatrixELementsToFile(const double *array, unsigned int numberOfRows, unsigned int numberOfCols, int precision, FILE *fptr)
{
   //A for loop will cycle through each index and write it to the specified file
   for (unsigned int i=0;i<numberOfRows;i++)
	{
      WriteDoubleRowElemementsToFile(array,numberOfCols,precision, fptr);
      
      //Starts a new row for the next series (7 values) of numbers
      WriteStringToFile("\n",fptr);
	}
   fclose(fptr);
   return 0;


}
//-----------------------------------------------------------------------------------------------------------
bool WriteDoubleRowElemementsToFile(const double *array, unsigned int numberOfCols, int precision, FILE *fptr)
{
   //static counter is used to increment the row index
   static int columnCounter=0;

   //counter to keep track of index
	int addToArray4Write=(numberOfCols*columnCounter);
	
   //variable to hole fieldwidth
   int fieldwidth=precision+8;
   
   //For loop to write each index (n=35) to the file specified by calling function
   for(unsigned int j=0;j<numberOfCols;j++)    
   {
      //Function that writes index to file in specified format
      WriteDoubleToFileWithExponentInSpecifiedPrecision(array[j+addToArray4Write], precision, fptr);
   }
	
   // next line increments static counter
   columnCounter=columnCounter+1;
   return 0;


}
//------------------------------------------------------------------------------------------------------
bool GetDoubleMatrixElementsFromFile(double *array, unsigned int numberOfRows, unsigned int numberOfCols, FILE *fptr)
{
   //For loop that will pull each value for specified file
   for(unsigned int i=0; i<numberOfRows; i++) 
	{
      //Function to retrieve row elements from specified file
      GetDoubleRowElementsFromFile(array,numberOfCols,fptr);
	}
   
   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;
	

}
//------------------------------------------------------------------------------------------------------
bool GetDoubleRowElementsFromFile(double *array, unsigned int numberOfCols, FILE *fptr)
{
   //Static counter to increment the row index
   static int counterToGetToNextRow=0;

   //counter that increments through the rows
	int addToArray=(numberOfCols*counterToGetToNextRow);
	
   //For loop that increments through all the indicies of the N=35 matrix

   for(unsigned int j=0; j<numberOfCols; j++)
	{
      //Built in function that will read in numeric values of specified file
      fscanf(fptr,"%lg",array+j+addToArray);
	}

   //increments counter for next row
   counterToGetToNextRow=counterToGetToNextRow+1;
   
   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;


}
//__________________________________________________________________________________

void AddMatrics(const double *arrayA, const double *arrayB, double *arraySum, unsigned int numberOfElements)
{
	
   //function adds two matrices together
   for(unsigned int i=0;i<numberOfElements;i++)
   {
         //next line with sum each index of specified matrix
         arraySum[i]=(arrayA[i]+arrayB[i]);
   }
}


//____________________________________________________________________________________________

bool GetStringFromKeyboard( char inputString[], unsigned long sizeOfString)
{ 
	//Returns the users input from the keyboard to the character inputString
   GetStringFromFile(inputString,256,stdin);
	 return 0;
}   


//_________________________________________________________________________________
bool GetStringFromFile(char inputString[], unsigned long sizeOfString, FILE* fptr)
{
	//Returns the users input from the keyboard and stores it in the inputString
   fgets (inputString, sizeOfString, fptr );
	return 0;
}


//_____________________________________________________________________________________
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue)
{
   
	// Default return value (in case the string is not a valid number)
   returnValue = defaultValue;
   
   // Check if  s  is a NULL string or  "abc"  or  "123huh"  or  "(&junk#") and if not convert string to double 
   if (s)
   {
      char *pointerToCharacterAfterNumber = NULL;
      long x =strtol(s,&pointerToCharacterAfterNumber,0);
      char characterAfterNumber = pointerToCharacterAfterNumber ? *pointerToCharacterAfterNumber : 'z';
      if ( characterAfterNumber == '\0' || isspace (characterAfterNumber) )
      {
         returnValue = x;
			return pointerToCharacterAfterNumber;
      }
   }
   return NULL;
}


//___________________________________________________________________________________________
bool  WriteStringToScreen ( const char outputString[])
{
	//Writes the string input from outputString to file, in this case the screen.
   WriteStringToFile(outputString, stdout);
	return 0;
}


//___________________________________________________________________________________
bool  WriteStringToFile( const char outputString[], FILE *fptr)
{
	//Writes the string input from outputString to file, in this case the screen.
   fprintf(fptr,outputString);
	return 0;
}


//___________________________________________________________________________________________
bool  WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr )
{
   // Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
   // Next, calculate the field width so it includes one extra space to the right of the number.
   if( precision < 0 || precision > 17 ) precision = 15;
   int fieldWidth = precision + 8;

   // Create the format specifier and print the number
   char format[20];
   sprintf( format, " %%- %d.%dE", fieldWidth, precision );
   
   //Returns a error message if unable to write double
   return fprintf( fptr, format, x) >0;
}


//---------------------------------------------------------------------
FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute)

//Open files for writing and if not, returns an error message. 
{
	FILE *p2File;
	p2File=fopen(filename,attribute);
	if (p2File==NULL)
	{
		WriteStringToScreen("Sorry, could not open file that was requested, please check name and re-enter\n");
		return NULL;
	}
	return p2File;
}


//__________________________________________________________________________________________________________
